class AmphibianMgr
	attr_accessor :node

	def initialize(sockets)
		@amphsock = nil
		@conns = {}
		@conn_from_id = {}
		@id_from_conn = {}
		@conn_from_port = {}
		@port_from_conn = {}
		@awaiting_read = {}

		@sockets = sockets
	end

	def start(addr, port)
		puts "starting up amphibian on #{addr}:#{port}"
		@amphsock = TCPServer.new(addr, port)
		@sockets << @amphsock
	end

	def process(source)
		if source == @amphsock
			new_conn
			return true
		elsif @conns.has_key? source
			amph_try(source) { process_in(source) }
			return true
		end
		return false
	end

	def new_conn
		newconn = @amphsock.accept
		puts "new conn " + newconn.inspect
		process_in(newconn)
		@sockets << newconn
		@conns[newconn] = true
	end

	def throw_away(conn)
		@sockets.delete(conn)
		@conns.delete(conn)
	end

	def amph_try(conn)
		begin
			yield
		rescue Errno::EPIPE
			puts "#{amphsock} is gone"
			throw_away(conn)
		end
	end

	def do_read(amphconn, len)
		id = @id_from_conn[amphconn]
		data = @node.transport_read(id, len)
		if data and data.length > 0
			amphconn.puts data.length
			amphconn.print data + "\n"
			@awaiting_read.delete(id)
		else
			@awaiting_read[id] = len
		end
	end

	def process_in(amphconn)
		cmdline = amphconn.gets
		unless cmdline and cmdline.length > 0
			throw_away(amphconn)
			return
		end
		puts "amph cmd: #{cmdline}"
		cmd, args = cmdline.split(/ /, 2)
		args = args.split(/ /).map{|x| x.to_i} if args

		case cmd
		when /^connect/
			addr, port = args
			id = @node.transport_connect(addr, port)
			@conn_from_id[id] = amphconn
			@id_from_conn[amphconn] = id
		when /^read/
			len = args[0]
			do_read(amphconn, len)
		when /^w(rite)?/
			length = args[0]
			puts "req to write #{length} bytes of data"
			data = amphconn.read(length+1).chomp
			puts "req to write data #{data.inspect}"
			# read out a \r\n if there was one
			#amphconn.read(1) if data[-1] == '\r'[0]
			id = @id_from_conn[amphconn]
			@node.transport_write(id, data)
			amphconn.puts "written"
	 	when /^b(ind)?/
	 		port = args[0]
	 		if @node.transport_bind(port)
				@conn_from_port[port] = amphconn
				@port_from_conn[amphconn] = port
	 			amphconn.puts 'bound'
	 		else
	 			amphconn.puts 'bind failed'
	 		end
		when /^a(ccept)?/
			port = @port_from_conn[amphconn]
			@node.transport_accept(port)
			# block!
		when /^receiver?/
			id = args[0]
			@conn_from_id[id] = amphconn
			@id_from_conn[amphconn] = id
			amphconn.puts "receiver success"
		when /^close/
			id = @id_from_conn[amphconn]
			@node.transport_close(id)
		else
			puts "UNKNOWN COMMAND: #{cmd}"
		end
	end

	def transport_connected(id)
		puts "conn id #{id} connected"
		amphconn = @conn_from_id[id]
		return unless amphconn
		amph_try(amphconn) {
			amphconn.puts "connected"
		}
	end

	def transport_accepted(port, id, remoteaddr, remoteport)
		puts "port #{port} accepted conn from #{remoteaddr}:#{remoteport}, newsock #{id}"
		amphconn = @conn_from_port[port]
		return unless amphconn
		amph_try(amphconn) {
			amphconn.puts "#{id} #{remoteaddr} #{remoteport}"
		}
	end

	def transport_readready(id)
		puts "amph: read ready on #{id}"
		if @awaiting_read[id]
			puts "amph: reading on #{id}"
			do_read(@conn_from_id[id], @awaiting_read[id])
		end
	end
end

# vim: set ts=2 sw=2 noet :
